#!/usr/bin/env python3
from enum import Enum
from typing import Optional
from telegram import ReplyKeyboardMarkup, User

from home.config import config
from home.telegram import bot
from home.telegram._botutil import user_any_name
from home.relay.sunxi_h3_client import RelayClient
from home.api.types import BotType
from home.mqtt import MqttNode, MqttModule, add_mqtt_module

config.load('pump_bot')

mqtt: Optional[MqttNode] = None
mqtt_relay_module: Optional[MqttModule] = None

bot.initialize()
bot.lang.ru(
    start_message="Выберите команду на клавиатуре",
    unknown_command="Неизвестная команда",

    enable="Включить",
    enable_silently="Включить тихо",
    enabled="Включен ✅",

    disable="Выключить",
    disable_silently="Выключить тихо",
    disabled="Выключен ❌",

    start_watering="Включить полив",
    stop_watering="Отключить полив",

    status="Статус",
    done="Готово 👌",
    user_action_notification='Пользователь <a href="tg://user?id=%d">%s</a> <b>%s</b> насос.',
    user_watering_notification='Пользователь <a href="tg://user?id=%d">%s</a> <b>%s</b> полив.',
    user_action_on="включил",
    user_action_off="выключил",
    user_action_watering_on="включил",
    user_action_watering_off="выключил",
)
bot.lang.en(
    start_message="Select command on the keyboard",
    unknown_command="Unknown command",

    enable="Turn ON",
    enable_silently="Turn ON silently",
    enabled="Turned ON ✅",

    disable="Turn OFF",
    disable_silently="Turn OFF silently",
    disabled="Turned OFF ❌",

    start_watering="Start watering",
    stop_watering="Stop watering",

    status="Status",
    done="Done 👌",
    user_action_notification='User <a href="tg://user?id=%d">%s</a> turned the pump <b>%s</b>.',
    user_watering_notification='User <a href="tg://user?id=%d">%s</a> <b>%s</b> the watering.',
    user_action_on="ON",
    user_action_off="OFF",
    user_action_watering_on="started",
    user_action_watering_off="stopped",
)


class UserAction(Enum):
    ON = 'on'
    OFF = 'off'
    WATERING_ON = 'watering_on'
    WATERING_OFF = 'watering_off'


def get_relay() -> RelayClient:
    relay = RelayClient(host=config['relay']['ip'], port=config['relay']['port'])
    relay.connect()
    return relay


def on(ctx: bot.Context, silent=False) -> None:
    get_relay().on()
    ctx.reply(ctx.lang('done'))
    if not silent:
        notify(ctx.user, UserAction.ON)


def off(ctx: bot.Context, silent=False) -> None:
    get_relay().off()
    ctx.reply(ctx.lang('done'))
    if not silent:
        notify(ctx.user, UserAction.OFF)


def watering_on(ctx: bot.Context) -> None:
    mqtt_relay_module.switchpower(mqtt, True, config.get('mqtt_water_relay.secret'))
    ctx.reply(ctx.lang('done'))
    notify(ctx.user, UserAction.WATERING_ON)


def watering_off(ctx: bot.Context) -> None:
    mqtt_relay_module.switchpower(mqtt, False, config.get('mqtt_water_relay.secret'))
    ctx.reply(ctx.lang('done'))
    notify(ctx.user, UserAction.WATERING_OFF)


def notify(user: User, action: UserAction) -> None:
    notification_key = 'user_watering_notification' if action in (UserAction.WATERING_ON, UserAction.WATERING_OFF) else 'user_action_notification'
    def text_getter(lang: str):
        action_name = bot.lang.get(f'user_action_{action.value}', lang)
        user_name = user_any_name(user)
        return 'ℹ ' + bot.lang.get(notification_key, lang,
                                   user.id, user_name, action_name)

    bot.notify_all(text_getter, exclude=(user.id,))


@bot.handler(message='enable')
def enable_handler(ctx: bot.Context) -> None:
    on(ctx)


@bot.handler(message='enable_silently')
def enable_s_handler(ctx: bot.Context) -> None:
    on(ctx, True)


@bot.handler(message='disable')
def disable_handler(ctx: bot.Context) -> None:
    off(ctx)


@bot.handler(message='start_watering')
def start_watering(ctx: bot.Context) -> None:
    watering_on(ctx)


@bot.handler(message='stop_watering')
def stop_watering(ctx: bot.Context) -> None:
    watering_off(ctx)


@bot.handler(message='disable_silently')
def disable_s_handler(ctx: bot.Context) -> None:
    off(ctx, True)


@bot.handler(message='status')
def status(ctx: bot.Context) -> None:
    ctx.reply(
        ctx.lang('enabled') if get_relay().status() == 'on' else ctx.lang('disabled')
    )


@bot.defaultreplymarkup
def markup(ctx: Optional[bot.Context]) -> Optional[ReplyKeyboardMarkup]:
    buttons = [
        [ctx.lang('enable'), ctx.lang('disable')],
    ]

    if ctx.user_id in config['bot']['silent_users']:
        buttons.append([ctx.lang('enable_silently'), ctx.lang('disable_silently')])

    buttons.append([ctx.lang('status')])
    buttons.append([ctx.lang('start_watering'), ctx.lang('stop_watering')])

    return ReplyKeyboardMarkup(buttons, one_time_keyboard=False)


if __name__ == '__main__':
    mqtt = MqttNode(node_id=config.get('mqtt_water_relay.node_id'))
    mqtt_relay_module = add_mqtt_module(mqtt, 'relay')

    bot.enable_logging(BotType.PUMP)
    bot.run()
